package com.kupal.commons.cs.impl;

import com.kupal.commons.cda.Dao;
import com.kupal.commons.cdm.Persistence;
import com.kupal.commons.cs.Service;
import com.kupal.commons.cs.exception.ServiceException;
import org.apache.commons.collections.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Krauchenia Sergey
 *         Date: 21.07.2010
 *         Time: 15:31:37
 */
public abstract class AbstractService<T extends Persistence> implements Service<T> {

    protected abstract Dao<T> getDao();

    @Override
    public void save(Persistence object) throws ServiceException {
        getDao().save(object);
    }

    @Override
    public List<T> search(T example) throws ServiceException {
        return getDao().search(example);
    }

    @Override
    public T update(T object) throws ServiceException {
        return getDao().update(object);
    }

    @Override
    public T saveOrUpdate(T object) throws ServiceException {
        return getDao().saveOrUpdate(object);
    }

    @Override
    public T merge(T object) throws ServiceException {
        return getDao().merge(object);
    }

    @Override
    public T saveOrMerge(T object) throws ServiceException {
        return getDao().saveOrMerge(object);
    }

    @Override
    public void delete(Persistence object) throws ServiceException {
        getDao().delete(object);
    }

    @Override
    public T load(long uidPk) throws ServiceException {
        return getDao().load(uidPk);
    }

    @Override
    public T get(long uidPk) throws ServiceException {
        return getDao().get(uidPk);
    }

    @Override
    public List<T> find(List propertyCriteria) throws ServiceException {
        return getDao().find(propertyCriteria);
    }

    @Override
    public List<T> bulkUpdate(List<T> objects) throws ServiceException {
        if(CollectionUtils.isEmpty(objects))
            return new ArrayList<T>(0);

        List<T> updated = new ArrayList<T>(0);
        for (T object : objects) {
            updated.add(getDao().update(object));
        }
        return updated;
    }

    @Override
    public List<T> list() throws ServiceException {
        return getDao().list();
    }
}
